use Test::More tests => 8;

use_ok 'Rex::Inventory::SMBios::BaseBoard';
use_ok 'Rex::Inventory::SMBios::Bios';
use_ok 'Rex::Inventory::SMBios::CPU';
use_ok 'Rex::Inventory::SMBios::Memory';
use_ok 'Rex::Inventory::SMBios::MemoryArray';
use_ok 'Rex::Inventory::SMBios::Section';
use_ok 'Rex::Inventory::SMBios::SystemInformation';
use_ok 'Rex::Inventory::SMBios';

